package zero1hd.polyjet.audio.map;

import org.apache.commons.math3.random.MersenneTwister;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.utils.FloatArray;

import zero1hd.polyjet.Main;
import zero1hd.polyjet.audio.AudioAnalyzer;
import zero1hd.polyjet.entity.EntityIndex;
import zero1hd.polyjet.util.MiniEvents;
import zero1hd.polyjet.util.MiniSender;

public class RhythmMapAlgorithm implements Runnable {
	private MiniSender sender;
	
	private FloatArray bassPeaks;
	private FloatArray UMPeaks;
	private FloatArray overlappedPeaks;
	private MersenneTwister rand;
	private GamePlayMap map;
	
	private float avgBass;
	private float avgUM;
	private float avgSPB;
	private float umMax, bassMax;
	
	private float speedMod, healthMod, difficultyMod;
	
	private float windowPerSecond;
	
	private volatile int progress;
	public RhythmMapAlgorithm(AudioAnalyzer analyzer, float speedMod, float healthMod, float difficultyMod) {
		sender = new MiniSender();
		
		bassPeaks = analyzer.getBassPeaks();
		UMPeaks = analyzer.getUMPeaks();
		overlappedPeaks = analyzer.getOverlappedPeaks();
		map = new GamePlayMap(analyzer.getAudioData());
		rand = new MersenneTwister(analyzer.getPUID());
		avgSPB = analyzer.getAvgSPB();
		windowPerSecond = 1/analyzer.getsecondsPerWindow();
		
		this.speedMod = speedMod;
		this.healthMod = healthMod;
		this.difficultyMod = difficultyMod;
		
		umMax = analyzer.getUMMaxValue();
		avgUM = analyzer.getUmAvg();

		bassMax = analyzer.getBassMaxValue();
		avgBass = analyzer.getBassAvg();
	}
	
	
	EntitySpawnInfo esi;
	@Override
	public void run() {
		map.beginBuild();
		for (int index = 0; index < bassPeaks.size; index++) {
			if (bassPeaks.get(index) != 0 || UMPeaks.get(index) != 0) {
				if (bassPeaks.get(index) != 0) {
					//If there is a value of some sorts that is not zero, we generate some beat for the map
					if (bassPeaks.get(index) >= avgBass) {
						//If bass peak is greater than the bass peak average, then:
						int indexMoved = map.goBack((int) (windowPerSecond*1.5f));
						float waitTime = indexMoved/windowPerSecond;
						float endRadius = (bassPeaks.get(index)/bassMax)*(Main.GAME_AREA_HEIGHT/4f);
						
						esi = map.nextEntitySpawnInfo(EntityIndex.VOID_CIRCLE);
						esi.parameters.put("warningTime", waitTime);
						esi.parameters.put("endRadius", endRadius);
						esi.parameters.put("growthRate", endRadius/(avgSPB*0.7f));
						esi.parameters.put("x", rand.nextFloat()*Main.GAME_AREA_WIDTH);
						esi.parameters.put("y", rand.nextFloat()*Main.GAME_AREA_HEIGHT);
						
						map.goForward(indexMoved);
					}
				}
				
				if (UMPeaks.get(index) != 0) {
					if (UMPeaks.get(index) >= avgUM) {
						//If upper midrange peaks are greater than average, the:
						int spawnLocations = (Main.GAME_AREA_WIDTH-8)/8;
						
						esi = map.nextEntitySpawnInfo(EntityIndex.BAR);
						esi.parameters.put("x", (float) (MathUtils.round(rand.nextFloat()*spawnLocations)*8));
						esi.parameters.put("rate", (8f/avgSPB)*speedMod);
						
					} else {
						float xSpawnLocation = (rand.nextFloat()*(Main.GAME_AREA_WIDTH-2))+1;
						
						esi = map.nextEntitySpawnInfo(EntityIndex.PELLET);
						esi.parameters.put("x", xSpawnLocation);
						esi.parameters.put("y", Main.GAME_AREA_HEIGHT-0.25f);
						esi.parameters.put("angle", 140*rand.nextFloat()+110f);
						esi.parameters.put("speed", (Main.GAME_AREA_HEIGHT/4f)/avgSPB);
						
					}
				}
				
				if (overlappedPeaks.get(index) != 0) {
					
				}
				
				if (rand.nextFloat() < 0.15f) {
					switch(rand.nextInt(10)) {
					case 0:
						break;
					}
				}
			} else {
				map.addNullToMap();
			}
			map.nextWindow();

			progress = MathUtils.round(100f*index/bassPeaks.size);
			
			sender.send(MiniEvents.MAPGEN_ITERATED);
		}
		map.endBuild();
		sender.send(MiniEvents.MAP_GENERATED);
	}
	
	public synchronized GamePlayMap getMap() {
		return map;
	}
	
	public synchronized int getProgress() {
		return progress;
	}
	
	public MiniSender getSender() {
		return sender;
	}
}
