package zero1hd.polyjet.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.ParticleEffect;
import com.badlogic.gdx.graphics.g2d.ParticleEffectPool;
import com.badlogic.gdx.graphics.g2d.ParticleEffectPool.PooledEffect;
import com.badlogic.gdx.utils.Array;

import zero1hd.polyjet.Main;

public class CollisionDetector {
	Array<Entity> firstGroup;
	Array<Entity> secondGroup;
	
	AssetManager assets;
	Preferences prefs;
	//Particle pools;
	ParticleEffectPool explosionEffectPool;
	Array<PooledEffect> effects = new Array<>();
	
	Sound explosionSFX;
	public CollisionDetector(Array<Entity> firstGroup, Array<Entity> secondGroup, AssetManager assetManager, Preferences prefs) {
		this.firstGroup = firstGroup;
		this.secondGroup = secondGroup;
		assets = assetManager;
		this.prefs = prefs;
		
		explosionEffectPool = new ParticleEffectPool(assets.get("explosion-s.p", ParticleEffect.class), 1, 64);
		explosionSFX = assets.get("explosion.ogg", Sound.class);
	}
	
	public void collisionCheck() {
		if ((firstGroup.size != 0) && (secondGroup.size != 0)) {
			for (int f = 0; f < firstGroup.size; f++) {
				Entity fe = firstGroup.get(f);
				for (int s = 0; s < secondGroup.size; s++) {
					Entity se = secondGroup.get(s);
					
					if (fe.getHitZone().overlaps(se.getHitZone())) {
						Gdx.app.debug("Collision Detector", "Collision between entities: " + fe.getEntityType() + " and " + se.getEntityType());
						
						//Play FX;
						if (se.playCollideSFX() && fe.playCollideSFX()) {
							explosionSFX.play(prefs.getFloat("fx vol"), 1f, (fe.getX()/Main.GAME_AREA_WIDTH)-0.55f);
						}
						if (se.playCollidePFX() && fe.playCollidePFX()) {
							PooledEffect currentPFX;
							currentPFX = explosionEffectPool.obtain();
							currentPFX.setPosition(fe.getX() + fe.getWidth()/2f, fe.getY() + fe.getHeight()/2f);
							effects.add(currentPFX);
						}
						
						fe.collided(se);
						se.collided(fe);
						break;
					}
				}
			}
		}
	}
	
	
	/**
	 * draws the explosion particles where necessecary.
	 * @param batch the batch used to draw the said particles
	 * @param cleanBatch whether this method should call begin and end on the batch.
	 */
	public void renderParticles(Batch batch, float delta, boolean cleanBatch) {
		if (cleanBatch) {
			batch.begin();
		}
		for (int i = 0; i < effects.size; i++) {
			effects.get(i).draw(batch, delta);
			if (effects.get(i).isComplete()) {
				effects.get(i).free();
				effects.removeIndex(i);
			}
		}
		
		if (cleanBatch) {
			batch.end();
		}
	}
}
