package zero1hd.polyjet.entity.enemies;

import java.util.HashMap;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Rectangle;

import zero1hd.polyjet.Main;
import zero1hd.polyjet.entity.EntityIndex;
import zero1hd.polyjet.entity.Entity;
import zero1hd.polyjet.entity.EntityController;

public class Flake extends Entity {
	private float timer;
	private float totalTime;
	private Shard[] shards;
	private EntityController ec;
	
	@Override
	public void preInit() {
		sprite.setTexture(assets.get("flake.png", Texture.class));
		simple = true;
		enemy = true;
		setSize(3f, 3f);
		super.preInit();
	}
	
	public void init(float x, float y, float rate, float fuse, float angle, int shardCount) {
		this.shards = new Shard[shardCount];
		for (int i = 0; i < shards.length; i++) {
			shards[i] = ec.shard.buildEntity();
			shards[i].init(x, y, 360/shards.length*i, 0, 2);
			ec.getStage().addActor(shards[i]);
		}
		setSize(3f, 3f);
		this.speed = rate;
		this.timer = fuse;
		this.totalTime = fuse;
		this.angle = angle;
		hitbox.setSize(getWidth(), getHeight());
		setPosition(x-center.x, y-center.y);
	}
	
	/**
	 * params: shardCount, x, y, rate, fuse, angle;
	 */
	@Override
	public void init(HashMap<String, Float> params) {
		this.shards = new Shard[params.get("shardCount").intValue()];
		for (int i = 0; i < shards.length; i++) {
			shards[i] = ec.shard.buildEntity();
			shards[i].init(params.get("x"), params.get("y"), 360/shards.length*i, 0, 2);
			ec.getStage().addActor(shards[i]);
		}
		this.speed = params.get("rate");
		this.timer = params.get("fuse");
		this.totalTime = timer;
		this.angle = params.get("angle");
		hitbox.setSize(getWidth(), getHeight());
		setPosition(params.get("x")-getWidth()/2f, params.get("y")-getHeight()/2f);
		super.init(params);
	}
	
	@Override
	public void act(float delta) {
		if (timer > 0) {
			timer -= delta;
		}
		for (int i = 0; i < shards.length; i++) {
			shards[i].setPosition(center.x-shards[i].getCenter().x, center.y-shards[i].getCenter().y);
		}
		
		if (getX() > Main.GAME_AREA_WIDTH || getY() > Main.GAME_AREA_HEIGHT || getX() < 0-getWidth() || getY() < 0-getHeight()) {
			timer = 0;
		}
		super.act(delta);
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		sprite.setColor(0.1f,0.1f,0.1f,1f - timer/totalTime);
		super.draw(batch, parentAlpha);
	}
	
	@Override
	public void collided(Entity entity) {
		switch (entity.getEntityType()) {
		case LASER:
			timer --;
			break;
		default:
			break;
		}
	}

	@Override
	public Rectangle getHitZone() {
		return hitbox;
	}

	@Override
	public EntityIndex getEntityType() {
		return EntityIndex.FLAKE;
	}

	@Override
	public boolean isDead() {
		if (timer <= 0) {
			for (int i = 0; i < shards.length; i++) {
				shards[i].setSpeed(45f);
			}
			return true;
		} else {
			return false;
		}
	}

	@Override
	public void reset() {
		timer = 0;
		shards = null;
		hitbox.set(0, 0, 0, 0);
		totalTime = 0;
	}

}
