package zero1hd.polyjet.entity.enemies;

import java.util.HashMap;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.Rectangle;

import zero1hd.polyjet.Main;
import zero1hd.polyjet.entity.EntityIndex;
import zero1hd.polyjet.entity.Entity;

public class Shard extends Entity {
	private int hp;
	private int maxHp;
	
	@Override
	public void preInit() {
		sprite = new Sprite(assets.get("shard.png", Texture.class));
		sprite.setSize(2f, 3f);
		simple = false;
		enemy = true;
		super.preInit();
	}
	
	public void init(float x, float y, float angle, float rate, int hp) {
		speed = rate;
		this.hp = hp;
		maxHp = hp;
		this.angle = angle;
		sprite.setRotation(angle+90);
		setSize(2f, 2f);
		setPosition(x-(getWidth()/2f), y-(getHeight()/2f));
		hitbox.setSize(getWidth(), getHeight());
		sprite.setOriginCenter();
	}
	
	/**
	 * x, y, angle, rate, hp;
	 */
	@Override
	public void init(HashMap<String, Float> params) {
		setX(params.get("x"));
		setY(params.get("y"));
		angle = params.get("angle");
		speed = params.get("rate");
		hp = params.get("hp").intValue();
		super.init(params);
	}
	
	@Override
	public void reset() {
		hp = 0;
		maxHp = 0;
	}
	
	@Override
	public void act(float delta) {
		
		if (getX() > Main.GAME_AREA_WIDTH || getY() > Main.GAME_AREA_HEIGHT || getX() < 0-getWidth() || getY() < 0-getHeight()) {
			hp = 0;
		}
		super.act(delta);
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		sprite.setColor(((float)hp/(float)maxHp), ((float)hp/(float)maxHp), ((float)hp/(float)maxHp), 0.5f);
		sprite.draw(batch);
		batch.setColor(Color.WHITE);
		super.draw(batch, parentAlpha);
	}
	
	@Override
	public void collided(Entity entity) {
		switch (entity.getEntityType()) {
		case LASER:
			hp--;
			break;
		default:
			break;
		}
	}

	@Override
	public Rectangle getHitZone() {
		return hitbox;
	}

	@Override
	public EntityIndex getEntityType() {
		return EntityIndex.SHARD;
	}

	@Override
	public boolean isDead() {
		if (hp <= 0) {
			return true;
		}
		return false;
	}
}
